from vcCommand import *
import xml.etree.ElementTree as ET
import os.path
app = getApplication()

def first_state():
  # FIRST STATE CREATES THE UI FOR THE USER
  global group, uri_xml, uri_csv,cvsCompList, vcidDict, lookForProps
  global add_line_c_prop, remove_line_c_prop, generate_prop, write_xml_prop, read_xml_prop
  removeAllLineComponents( )
  # REMOVE EXISTING COMMAND PROPERTIES
  cmd = getCommand()
  cmdProps = cmd.Properties
  for p in cmdProps:
    deleteProperty(p)
  # READ URI
  uri = getCommandPath()
  base, filename = os.path.split(uri)
  # uri for the csv file
  uri_csv = os.path.join(base, 'data_component_vcid.csv')
  uri_csv = uri_csv[8:]
  # uri for the XML file
  uri_xml = os.path.join(base, 'Line_XML.xml')
  uri_xml = uri_xml[8:]
  # IMPORT CSV DATA
  cvsCompList = []
  vcidDict = {}
  with open(uri_csv, 'r') as csv_file:
      content = csv_file.read()
      lines = content.split('\n')
      for line in lines:
        if ';' in line:
          comp_csv,vcid = line.split(';')
          #print comp_csv,vcid
          cvsCompList.append(comp_csv)
          vcidDict[comp_csv] = vcid
        else: 
          break
  # CREATE PROPERTIES
  group = 0
  group2 = 0
  # ADD LINE COMPONENT BUTTON
  add_line_c_prop = createProperty(VC_BUTTON, 'Add Line Component')
  add_line_c_prop.Group = group = group + 10
  add_line_c_prop.OnChanged = addLineComponent
  # REMOVE LINE COMPONENT BUTTON 
  remove_line_c_prop = createProperty(VC_BUTTON, 'Remove Line Component')
  remove_line_c_prop.Group = group = group + 10
  remove_line_c_prop.OnChanged = removeLineComponent
  # GENERATE
  generate_prop = createProperty(VC_BUTTON, 'Generate')
  generate_prop.Group = group2 = group2 + 500
  generate_prop.OnChanged = generate
  # WRITE to XML
  write_xml_prop = createProperty(VC_BUTTON, 'XML::Write to XML')
  write_xml_prop.Group = group2 = group2 + 500
  write_xml_prop.OnChanged = createXML
  # READ from XML
  read_xml_prop = createProperty(VC_BUTTON, 'XML::Generate from XML')
  read_xml_prop.Group = group2 = group2 + 500
  read_xml_prop.OnChanged = readXML
  #
  executeInActionPanel() 

def getLineProps():
  a = 1
  line_props = []
  while True:
    prop = getProperty('Comp%i' % a)
    a+=1
    if prop:
      line_props.append( prop )
    else:
      return line_props
      
def addLineComponent(arg): 
  global group
  line_props = getLineProps()
  prop_name = 'Comp'+str(len(line_props)+1)
  line_c = createProperty(VC_STRING, prop_name, VC_PROPERTY_STEP)
  line_c.StepValues = cvsCompList
  if line_props == []:
    line_c.Value = 'Loader'  
  else:
    line_c.Value = 'Unloader'
  line_c.IsVisible = True
  line_c.Group = group = group + 10
  line_props.append(line_c)
  # check if loader or unloader component between the line and change it to conveyor
  for prop in line_props[1:-1]:
    if prop.Value in ['Loader','Unloader']:
      prop.Value = 'Conveyor'
  executeInActionPanel() 

def removeLineComponent(arg):
  line_props = getLineProps()
  if line_props:
    line_props[-1].IsVisible = False
    deleteProperty(line_props[-1])

def removeAllLineComponents():
  for c in getLineProps():
    deleteProperty(c) 

def generate(arg):
  line_props = getLineProps()
  if checkLineErrors(line_props):
    createXML(False)
    readXML(False)

def createXML(arg):
  global uri_xml
  line_props = getLineProps()
  # ERROR CHECK
  if not checkLineErrors(line_props):
    return
  # CREATE XML 
  root = ET.Element("LineComponents")
  for i in range(len(line_props)):
    key = 'Comp'+ str(i+1)
    item = line_props[i].Value
    sub_e = ET.SubElement(root, key)
    sub_e.set('Type', item)
    ET.SubElement(sub_e,'VCID').text = vcidDict[item]
  tree = ET.ElementTree(root)
  indent(root)
  tree.write(uri_xml,encoding='utf-8', xml_declaration=True)
  if arg:
    print 'XML file written to:'
    print '  - %s' % uri_xml

def readXML(arg):
  '''# reads XML and generates the line '''
  if app.Components:
    ret = app.messageBox('Clear existing layout?', VC_MESSAGE_TYPE_QUESTION, VC_MESSAGE_BUTTONS_YESNO)
    if ret == VC_MESSAGE_RESULT_YES:
      app.clearWorld()
  prev_comp = None
  # READ XML
  tree = ET.parse(uri_xml)
  root = tree.getroot()
  #root = ET.fromstring()
  for elem in root:
    # elem.get('Type','')
    for subElem in elem:
      if subElem.tag == 'VCID':
        vcid = subElem.text
      else:
        print 'Error - VCID not found'
    # GENERATE COMPONENTS
    vcid = 'vcid:'+ vcid
    current_comp = app.load(vcid)
    if not prev_comp == None:
      app.connectComponents(prev_comp,current_comp)
    prev_comp = current_comp
  app.render()
  if arg:
    print 'Layout generated from XML file:'
    print '  - %s' % uri_xml

def checkLineErrors(line_props):
  passed = True
  if not line_props:
    print 'ERROR - Please define line components first.'
    return False
  if not line_props[0].Value == 'Loader':
    print 'ERROR - Line is not starting with LOADER'
    passed = False
  if not line_props[-1].Value == 'Unloader':
    print 'ERROR - Line is not ending with UNLOADER'
    passed = False
  for i in range(len(line_props)):
    if line_props[i].Value == 'Loader':
      if not i == 0:
        print 'ERROR - LOADER should be only the first component'
        passed = False
    if line_props[i].Value == 'Unloader':
      if not i == len(line_props)-1:   
        print 'ERROR - UNLOADER should be only the last component'
        passed = False
  return passed

def indent(elem, level=0):
  # INDENT XML
  i = "\n" + level*"  "
  if len(elem):
    if not elem.text or not elem.text.strip():
      elem.text = i + "  "
    if not elem.tail or not elem.tail.strip():
      elem.tail = i
    for elem in elem:
      indent(elem, level+1)
    if not elem.tail or not elem.tail.strip():
      elem.tail = i
  else:
    if level and (not elem.tail or not elem.tail.strip()):
      elem.tail = i

addState(first_state)